@@ -1,4 +1,4 @@
-from unittest import mock
+import dataclasses
 
 import fastapi
 import pytest
@@ -22,14 +22,28 @@ Ask Func[simple2]: Simple argument
 Func[simple2] says: Simple response
 A: Simple final response
 """
+CORPORA = [
+    agents.DocumentCorpus(
+        urls=["http://example.com/doc1.txt"], loader=agents.DocumentLoader("text")
+    ),
+]
+
+
+@pytest.fixture(autouse=True)
+def mock_token_verifier(mocker):
+    return mocker.patch.object(
+        code_shot._VerifiedTokenClaims,
+        "from_token",
+        return_value=code_shot._VerifiedTokenClaims(agent_id="fake agent id"),
+    )
 
 
 @pytest.fixture
 def dummy_agent():
     agent = agents.CodeShotAgent(
-        agent_id,
         BASE_PROMPT,
         FEW_SHOTS,
+        CORPORA,
         oauth_params=fixieai.OAuthParams(
             client_id="dummy",
             client_secret="dummy",
@@ -51,7 +65,6 @@ def dummy_agent():
     def simple3(query):
         return "Simple response custom"
 
-    agent._verify_token = mock.Mock(return_value=True)
     return agent
 
 
@@ -66,10 +79,11 @@ def test_simple_agent_handshake(dummy_agent):
     assert yaml_content == {
         "base_prompt": dummy_agent.base_prompt,
         "few_shots": dummy_agent.few_shots,
+        "corpora": [dataclasses.asdict(c) for c in dummy_agent.corpora],
     }
 
 
-def test_simple_agent_func_calls(dummy_agent):
+def test_simple_agent_func_calls(dummy_agent, mock_token_verifier):
     fast_api = fastapi.FastAPI()
     fast_api.include_router(dummy_agent.api_router())
     client = testclient.TestClient(fast_api, raise_server_exceptions=False)
@@ -82,7 +96,9 @@ def test_simple_agent_func_calls(dummy_agent):
     assert response.status_code == 200
     json = response.json()
     assert json == {"message": {"text": "Simple response 1", "embeds": {}}}
-    dummy_agent._verify_token.assert_called_once_with("fixie-test-token")
+    mock_token_verifier.assert_called_once_with(
+        "fixie-test-token", dummy_agent._jwks_client
+    )
 
     # Test Func[simple2]
     response = client.post(
@@ -241,3 +257,16 @@ def test_registering_func_with_custom_name(dummy_agent):
     assert dummy_agent._funcs["name1"] == good_typed_func1
     assert dummy_agent._funcs["name2"] == good_typed_func2
     assert dummy_agent._funcs["name3"] == good_typed_func3
+
+
+def test_invalid_token(dummy_agent, mock_token_verifier):
+    mock_token_verifier.return_value = None
+
+    fast_api = fastapi.FastAPI()
+    fast_api.include_router(dummy_agent.api_router())
+    client = testclient.TestClient(fast_api, raise_server_exceptions=False)
+    headers = {"Authorization": "Bearer fixie-test-token"}
+    response = client.post(
+        "/simple1", json={"message": {"text": "Howdy"}}, headers=headers
+    )
+    assert response.status_code == 403
