@@ -11,7 +11,7 @@ def session():
 @session.command("list", help="Lists sessions.")
 @click.pass_context
 def list_sessions(ctx):
-    client = ctx.obj["CLIENT"]
+    client = ctx.obj.client
     session_ids = client.get_sessions()
     for session_id in session_ids:
         click.secho(f"{session_id}", fg="green")
@@ -20,7 +20,7 @@ def list_sessions(ctx):
 @session.command("new", help="Creates a new session and opens it.")
 @click.pass_context
 def new_session(ctx):
-    client = ctx.obj["CLIENT"]
+    client = ctx.obj.client
     c = console.Console(client)
     c.run()
 
@@ -29,7 +29,7 @@ def new_session(ctx):
 @click.pass_context
 @click.argument("session_id")
 def open_session(ctx, session_id: str):
-    client = ctx.obj["CLIENT"]
+    client = ctx.obj.client
     session = client.get_session(session_id)
     messages = session.get_messages()
     click.echo(messages)
